#include <active_check.h>
#include <application_sleep.h>
#include <board_def.h>
#include <irq.h>
#include <rtc.h>

#define MAGIC_NUMBER 1836547290
static uint32_t magicNumber;
static uint32_t standbyTime;
static void (*callback)(void) = NULL;

/* Konfiguruj zegar czasu rzeczywistego. */
int RTCconfigure() {
  EXTI_InitTypeDef EXTI_InitStruct;
  NVIC_InitTypeDef NVIC_InitStruct;
  RTC_InitTypeDef  RTC_InitStruct;
  RTC_AlarmTypeDef RTC_AlarmStruct;

  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_BKPSRAM, ENABLE);
  PWR_BackupAccessCmd(ENABLE);

  if (PWR_GetFlagStatus(PWR_FLAG_SB) != RESET) {
    /* Mikrokontroler obudził się z trybu czuwania.
       Nie ma potrzeby ponownego konfigurowania RTC - konfiguracja
       jest zachowywana po wybudzeniu ze stanu czuwania. */
    PWR_ClearFlag(PWR_FLAG_SB);
    RTC_WaitForSynchro();
  }
  else {
    /* Mikrokontroler został wyzerowany. */

    /* Tu było BKP_DeInit(); co jest równowżne poniższej sekwencji. */
    RCC_BackupResetCmd(ENABLE);
    RCC_BackupResetCmd(DISABLE);

    /* Konfiguruj RTC, aby był taktowany kwarcem 32768 Hz. */
    RCC_LSEConfig(RCC_LSE_ON);
    active_check(RCC_GetFlagStatus(RCC_FLAG_LSERDY), 10000000);
    RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);
    RCC_RTCCLKCmd(ENABLE);
    RTC_WaitForSynchro();

    /* Konfiguruj RTC, aby tykał z okresem 1 s. */
    RTC_InitStruct.RTC_AsynchPrediv = 0x7F;
    RTC_InitStruct.RTC_SynchPrediv = 0xFF;
    RTC_InitStruct.RTC_HourFormat = RTC_HourFormat_24;
    if (RTC_Init(&RTC_InitStruct) == ERROR)
      return -1;

    /* Konfiguruj alarm RTC A zgłaszany co sekundę. */
    RTC_AlarmStruct.RTC_AlarmTime.RTC_Hours = 0;
    RTC_AlarmStruct.RTC_AlarmTime.RTC_Minutes = 0;
    RTC_AlarmStruct.RTC_AlarmTime.RTC_Seconds = 0;
    RTC_AlarmStruct.RTC_AlarmTime.RTC_H12 = 0;
    RTC_AlarmStruct.RTC_AlarmDateWeekDay = 0;
    RTC_AlarmStruct.RTC_AlarmDateWeekDaySel = 0;
    RTC_AlarmStruct.RTC_AlarmMask = RTC_AlarmMask_All;
    RTC_SetAlarm(RTC_Format_BIN, RTC_Alarm_A, &RTC_AlarmStruct);
    RTC_AlarmCmd(RTC_Alarm_A, ENABLE);

    /* Konfiguruj licznik pozwalający wybudzić mikrokontroler
       po zadanej liczbie sekund. */
    RTC_WakeUpClockConfig(RTC_WakeUpClock_CK_SPRE_17bits);
    RTC_WakeUpCmd(ENABLE);
  }

  /* Konfiguruj przerwanie RTC zgłaszane co sekundę. */
  RTC_ClearFlag(RTC_FLAG_ALRAF);
  EXTI_ClearITPendingBit(EXTI_Line17);
  EXTI_InitStruct.EXTI_Line = EXTI_Line17;
  EXTI_InitStruct.EXTI_Trigger = EXTI_Trigger_Rising;
  EXTI_InitStruct.EXTI_LineCmd = ENABLE;
  EXTI_InitStruct.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_Init(&EXTI_InitStruct);
  NVIC_InitStruct.NVIC_IRQChannelPreemptionPriority = LOW_IRQ_PRIO;
  NVIC_InitStruct.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStruct.NVIC_IRQChannelCmd = ENABLE;
  NVIC_InitStruct.NVIC_IRQChannel = RTC_Alarm_IRQn;
  NVIC_Init(&NVIC_InitStruct);

  /* Konfiguruj przerwanie budzika RTC. */
  RTC_ClearFlag(RTC_FLAG_WUTF);
  EXTI_ClearITPendingBit(EXTI_Line22);
  EXTI_InitStruct.EXTI_Line = EXTI_Line22;
  EXTI_Init(&EXTI_InitStruct);
  NVIC_InitStruct.NVIC_IRQChannel = RTC_WKUP_IRQn;
  NVIC_Init(&NVIC_InitStruct);

  return 0;
}

void RTCsetCallback(void (*f)(void)) {
  if (f == NULL) {
    /* Dezaktywuj przerwanie przed wyzerowaniem funkcji zwrotnej. */
    RTC_ITConfig(RTC_IT_ALRA, DISABLE);
  }

  callback = f;

  if (f) {
    /* Uaktywnij przerwanie zgłaszane co sekundę. */
    RTC_ITConfig(RTC_IT_ALRA, ENABLE);
  }
}

int GetRealTimeClock(struct tm *tm) {
  RTC_TimeTypeDef RTC_TimeStruct;
  RTC_DateTypeDef RTC_DateStruct;

  RTC_GetTime(RTC_Format_BIN, &RTC_TimeStruct);
  RTC_GetDate(RTC_Format_BIN, &RTC_DateStruct);

  tm->tm_sec   = RTC_TimeStruct.RTC_Seconds;
  tm->tm_min   = RTC_TimeStruct.RTC_Minutes;
  tm->tm_hour  = RTC_TimeStruct.RTC_Hours;
  tm->tm_mday  = RTC_DateStruct.RTC_Date;
  tm->tm_mon   = RTC_DateStruct.RTC_Month - 1;
  tm->tm_year  = RTC_DateStruct.RTC_Year + 100;
  tm->tm_wday  = RTC_DateStruct.RTC_WeekDay;
  tm->tm_yday  = 0;
  tm->tm_isdst = 0;
  if (tm->tm_wday == 7)
    tm->tm_wday = 0;
  if (tm->tm_sec < 0  || tm->tm_sec > 59  ||
      tm->tm_min < 0  || tm->tm_min > 59  ||
      tm->tm_hour < 0 || tm->tm_hour > 23 ||
      tm->tm_mday < 1 || tm->tm_mday > 31 ||
      tm->tm_mon < 0  || tm->tm_mon > 11  ||
      tm->tm_wday < 0 || tm->tm_wday > 6)
    return -1;
  else
    return 0;
}

int SetRealTimeClock(time_t t) {
  RTC_DateTypeDef RTC_DateStruct;
  RTC_TimeTypeDef RTC_TimeStruct;
  struct tm *tm;

  tm = gmtime(&t);
  if (tm == NULL)
    return -1;

  RTC_TimeStruct.RTC_Seconds = tm->tm_sec;
  RTC_TimeStruct.RTC_Minutes = tm->tm_min;
  RTC_TimeStruct.RTC_Hours   = tm->tm_hour;
  RTC_TimeStruct.RTC_H12     = 0;
  RTC_DateStruct.RTC_Date    = tm->tm_mday;
  RTC_DateStruct.RTC_Month   = tm->tm_mon + 1;
  RTC_DateStruct.RTC_Year    = tm->tm_year - 100;
  RTC_DateStruct.RTC_WeekDay = tm->tm_wday == 0 ? 7 : tm->tm_wday;

  if (RTC_SetDate(RTC_Format_BIN, &RTC_DateStruct) == ERROR)
    return -1;
  if (RTC_SetTime(RTC_Format_BIN, &RTC_TimeStruct) == ERROR)
    return -1;

  return 0;
}

/* Wejdź w stan czuwania na time sekund. */
void Standby(uint32_t time) {
  RTC_SetWakeUpCounter(time);
  RTC_ITConfig(RTC_IT_WUT, ENABLE);
  ApplicationSleep();
  PWR_EnterSTANDBYMode();
}

/* Wejdź w stan czuwania za delay sekund na time sekund. */
void DelayedStandby(uint32_t delay, uint32_t time) {
  if (delay > 0) {
    magicNumber = MAGIC_NUMBER;
    standbyTime = time;
    RTC_SetWakeUpCounter(delay);
    RTC_ITConfig(RTC_IT_WUT, ENABLE);
  }
  else {
    Standby(time);
  }
}

void RTC_Alarm_IRQHandler(void) {
  if (RTC_GetITStatus(RTC_IT_ALRA) != RESET)  {
    EXTI_ClearITPendingBit(EXTI_Line17);
    RTC_ClearITPendingBit(RTC_IT_ALRA);
    if (callback) {
      callback();
    }
  }
}

void RTC_WKUP_IRQHandler(void) {
  if (RTC_GetITStatus(RTC_IT_WUT) != RESET) {
    EXTI_ClearITPendingBit(EXTI_Line22);
    RTC_ClearITPendingBit(RTC_IT_WUT);
    RTC_ITConfig(RTC_IT_WUT, DISABLE);
    /* Sprawdź, czy jest to już wybudzenie, czy jednak
       dopiero teraz mikrokontroler ma usnąć. */
    if (magicNumber == MAGIC_NUMBER) {
      magicNumber = 0;
      Standby(standbyTime);
    }
  }
}
